/*

Copyright 2003 The University of New South Wales (UNSW) and National  
ICT Australia (NICTA).

This file is part of the 2003 team rUNSWift RoboCup entry.  You may  
redistribute it and/or modify it under the terms of the GNU General  
Public License as published by the Free Software Foundation; either  
version 2 of the License, or (at your option) any later version as  
modified below.  As the original licensors, we add the following  
conditions to that license:

In paragraph 2.b), the phrase "distribute or publish" should be  
interpreted to include entry into a competition, and hence the source  
of any derived work entered into a competition must be made available  
to all parties involved in that competition under the terms of this  
license.

In addition, if the authors of a derived work publish any conference  
proceedings, journal articles or other academic papers describing that  
derived work, then appropriate academic citations to the original work  
must be included in that publication.

This rUNSWift source is distributed in the hope that it will be useful,  
but WITHOUT ANY WARRANTY; without even the implied warranty of  
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  
General Public License for more details.

You should have received a copy of the GNU General Public License along  
with this source code; if not, write to the Free Software Foundation,  
Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

/*
 * Last modification background information
 * $Id: PWalk.h 1953 2003-08-21 03:51:39Z eileenm $
 *
 * Copyright (c) 2003 UNSW
 * All Rights Reserved.
 * 
 * contains the actual calculations used to execute the walking and kicking routines, handling the leg motion
**/

#ifndef _PWalk_h_DEFINED
#define _PWalk_h_DEFINED

#include <OPENR/OObject.h>
#include <OPENR/OSubject.h>
#include <OPENR/OObserverVector.h>
#include <OPENR/OPENREvent.h>
#include <OPENR/OPENRAPI.h>
#include <iostream>

#include "EffectorCommander.h"
#include "../share/Common.h"

#include "../share/PWalkDef.h"

#include "primitives.h"

#include "SpecialAction.h"

/* The number of walk locus end-points calculated per leg for each camera frame */
static const int numOfFrames = 5;

/* Robot body length constants */
static const double l1  = 64;
static const double l2  = 68.41286429;
static const double l3  = 12.8;
static const double lsh = 119;
static const double l4  = 75.92865072;


class PWalk {

public:

    OSubject * sEffector;

    PWalk (OSubject *, EffectorCommander &  /*, int *, bool * */  );
    ~PWalk () {
    	delete(chestpush);
		delete(fwdkick);
		delete(fastkick); 
		delete(getupf);
		delete(getups);
		delete(getupr);
		delete(backKick);
    }

    void pStep (
        int walktype, 
        
        /* specifies the distance in cm that the robot should move forward in each half-step
         * Negative values move the robot backwards
        **/
        double forward, // the length in cm to move forward per step
        
        /* specifies the distance in cm that the robot should move to the left in each half-step
         * Negative values move the robot backwards
        **/
        double left,    // the length in cm to move left per step
        
        /* specifies the angle in degrees that the robot should turn counter clockwise per half-step.
         * Negative values move the robot clockwise
        **/
        double turnCCW, // the angle in degrees to turn counter clockwise per step

        int speed,

        double hf,
        double hb,
        double hdf,
        double hdb,
        double ffo,
        double fso,
        double bfo,
        double bso
    );
    /* Note: * = this is actually not entirely true since there's a small bug in the PWalk code
     * where one of the formulae is not correctly translated;
     * the values of hdf and hdb actually have to be set to be a bit higher than their intended values
    **/

    void readLocusOffets(const char *filename);

    double delta_forward, delta_left, delta_turn;

    bool pwalkNotUsingHead;

    //int *leading_leg; /* -1 = left, 1 = right, 0 = unknown */
    //bool *stepComplete;
        
    SpecialAction *chestpush;
    SpecialAction *fwdkick;
    SpecialAction *fastkick;

    SpecialAction *getupf;
    SpecialAction *getups;
    SpecialAction *getupr;

    SpecialAction *backKick;
	SpecialAction *cheer;	

	//const pointers so that people on the outside can read 
	//walktype, step_, PG, but not write to it.
	const int* ptr_walktype;
	const int* ptr_step_;
	const int* ptr_PG;

private:

    double fc, sc, tc; /* forward, left & turn left control in mm */
    int step_;         /* one step_ per frame */
    
    /* height forward, the vertical height in mm from the shoulder joint to the ground
     * The point on the ground vertically below the shoulder joint is called "shoulder ground"
    **/
    double hf;
    
    /* height back, the vertical height in mm from the hip joint to the ground
     * The point on the ground vertically below the hip joint is called "hip ground"
    **/
    double hb;
    
    /* height delta forward, the height in mm that the robot lifts its front paws when walking,
     * that is, the height of the walk locus for the front paws.
    **/
    double hdf;
    
    /* height delta back, the height in mm that the robot lifts its back paws when walking,
     * that is, the height of the walk locus for the back paws.
    **/
    double hdb;

    /* front forward offset, home position of the front paws in mm forward from shoulder ground
    **/
    double ffo;
    
    /* front sideways offset, home position of the front paws in mm sideways from shoulder ground.
     * Larger values spread the legs outwards, smaller values bring them in closer
    **/
    double fso;
    
    /* back forward offset, home position of the back paws in mm forward from hip ground 
    **/
    double bfo;
    
    /* back sideways offset, home position of the back paws in mm sideways from hip ground.
     * Larger values spread the legs outwards, smaller values bring them in closer
    **/
    double bso;

    double hfG;
    double hbG;
    
    /* period ground, the time taken to complete a half-step 
     * (for the robot to move along the ground, or the bottom edge of the walk locus
     * PG = numOfFrames * number of camera frames per half-step
    **/
    int PG;            /* steps per ground stroke */
    
    double theta5, sin5, cos5;

    double jfl1, jfl2, jfl3, jfr1, jfr2, jfr3, jbl1, jbl2, jbl3, jbr1, jbr2, jbr3;

    /* specifies the actual type of locomotion action to execute. 
     * Walktypes are definitions used to decide whether to execute a particular walking style or kicking action.
     * (exception: aimable forward kick is a kicking action that ignore all other parameter values)
    **/
    int walktype;

    /* locus point offsets
     */
    double bfl1, bsl1, bhl1, cfl1, csl1, chl1, dfl1, dsl1, dhl1, efl1, esl1, ehl1;
    double bfl2, bsl2, bhl2, cfl2, csl2, chl2, dfl2, dsl2, dhl2, efl2, esl2, ehl2;

    static const int numOfBuffer = 2;
    OCommandVectorData *cmdVec_[numOfBuffer];
    MemoryRegionID cmdVecID_[numOfBuffer];
    RCRegion *cmdRegion[numOfBuffer];

    void SetUpBuffer (EffectorCommander &);
    slongword microRad (double);
    double abs (double);
    double microRad2Deg (slongword);

};

#endif // _PWalk_h_DEFINED
